
(*
 *	Copyright (c) 2002 by Laboratoire Spcification et Vrification (LSV),
 *	CNRS UMR 8643 & ENS Cachan.
 *	Written by Jean Goubault-Larrecq.  Not derived from licensed software.
 *
 *	Permission is granted to anyone to use this software for any
 *	purpose on any computer system, and to redistribute it freely,
 *	subject to the following restrictions:
 *
 *	1. Neither the author nor its employer is responsible for the consequences of use of
 *		this software, no matter how awful, even if they arise
 *		from defects in it.
 *
 *	2. The origin of this software must not be misrepresented, either
 *		by explicit claim or by omission.
 *
 *	3. Altered versions must be plainly marked as such, and must not
 *		be misrepresented as being the original software.
 *
 *	4. This software is restricted to non-commercial use only.  Commercial
 *		use is subject to a specific license, obtainable from LSV.
 *)

open Cparse
open Control
open Zones
open Sigs
open Glue

type abs_info = (loc_ppoint option, (loc_ppoint, abs_env) Hashtbl.t) Hashtbl.t
    (* envoie chaque point de programme vers un environnement abstrait.
     Si le point de programme pp n'est pas dans l'abs_info, ceci
     signifie qu'il est envoye vers l'environnement bottom (code mort). *)

type sig_info = (loc_ppoint option, (loc_instr, sig_set) Hashtbl.t) Hashtbl.t
    (* envoie chaque instruction (avec eventuellement un point de programme
     identifiant la ligne appelant la fonction courante) vers la liste
     des signaux pouvant etre lances par cette instruction.
     Si l'instruction i est envoyee vers une table de hash H, alors H est
     non vide.  De plus H envoie chaque contexte ctx possible vers une liste
     de signaux non vides.  Autrement dit, les cas ou les listes de signaux
     sont vides ne sont pas enregistres. *)

type function_info = (string, (loc_ppoint option,
			       abs_value AbsEnv.t option ref *
			       abs_value option ref *
			       abs_addr ref *
			       sig_set ref) Hashtbl.t) Hashtbl.t
    (* envoie chaque nom de fonction et chaque contexte d'appel
     vers:
     - un(e moitie d') environnement representant les valeurs des variables
     globales a la fin de la fonction, ou None si la fonction ne retourne pas;
     - une valeur abstraite representant les valeurs de retour possibles,
     ou None si la fonction ne retourne pas ou bien est de type de retour void;
     - une adresse abstraite representant tous les effets de bord
     que cette fonction effectue possiblement lorsqu'elle est appelee
     dans ce contexte;
     - un ensemble de signaux que l'appel a la fonction peut lancer.
     *)

type functions_not_found = (string, unit) Hashtbl.t
    (* ensemble de toutes les fonctions dont on ne connait pas le source
     et qui ont ete appelees par au moins une fonction dans le code analyse. *)

module CallContext : Set.S
(*
  sig
    type elt = Control.ppoint
    and t
    val empty : t
    val is_empty : t -> bool
    val mem : elt -> t -> bool
    val add : elt -> t -> t
    val singleton : elt -> t
    val remove : elt -> t -> t
    val union : t -> t -> t
    val inter : t -> t -> t
    val diff : t -> t -> t
    val compare : t -> t -> int
    val equal : t -> t -> bool
    val subset : t -> t -> bool
    val iter : (elt -> unit) -> t -> unit
    val fold : (elt -> 'a -> 'a) -> t -> 'a -> 'a
    val cardinal : t -> int
    val elements : t -> elt list
    val min_elt : t -> elt
    val max_elt : t -> elt
    val choose : t -> elt
  end
 *)

type abs_call_effect =
     (abs_value AbsEnv.t * abs_value option * abs_addr * sig_set) option
	(* L'effet abstrait d'un appel a une fonction:
	soit None si la fonction ne retourne pas,
	soit Some (globs, retval, a, sigs) ou
	globs est la moitie de l'environnement portant sur les valeurs
	des variables globales apres le retour de la fonction,
	retval est la valeur de retour (si pas de type void; sinon,
	retval = None),
	a est une adresse abstraite contenant toutes les adresses sur
	lesquelles l'appel a la fonction a pu faire des effets de bord,
	et sigs est l'ensemble des signaux possiblement lances par
	la fonction. *)

val external_functions :
	(string,
	var_table -> fun_table -> abs_info -> function_info ->
	sig_info -> functions_not_found -> heap_size_info ->
	CallContext.t -> ppoint -> abs_value AbsEnv.t -> abs_value list ->
	q_ctype -> abs_call_effect
	) Hashtbl.t

val verbose : int ref

val analyseur :
  var_table -> fun_table ->
  abs_info -> function_info -> sig_info -> functions_not_found -> heap_size_info ->
  CallContext.t -> loc_ppoint option -> abs_value AbsEnv.t ->
  string -> abs_value list ->
  q_ctype -> abs_call_effect
