
(*
 *	Copyright (c) 2002 by Laboratoire Spcification et Vrification (LSV),
 *	CNRS UMR 8643 & ENS Cachan.
 *	Written by Jean Goubault-Larrecq.  Not derived from licensed software.
 *
 *	Permission is granted to anyone to use this software for any
 *	purpose on any computer system, and to redistribute it freely,
 *	subject to the following restrictions:
 *
 *	1. Neither the author nor its employer is responsible for the consequences of use of
 *		this software, no matter how awful, even if they arise
 *		from defects in it.
 *
 *	2. The origin of this software must not be misrepresented, either
 *		by explicit claim or by omission.
 *
 *	3. Altered versions must be plainly marked as such, and must not
 *		be misrepresented as being the original software.
 *
 *	4. This software is restricted to non-commercial use only.  Commercial
 *		use is subject to a specific license, obtainable from LSV.
 *)


(* Projet no.7: Calculs d'equations semantiques correspondants aux fonctions
 de la bibliotheque standard C. *)

open Const
open Sigs
open Arith
open Error
open Intervalle
open Zones
open Glue
open Analyseur

(* Un exemple pour montrer comment on definit les fonctions.
 Ici on definit l'effet de malloc().
 Noter que malloc (n) a pour effet de:
 - retourner un pointeur vers Z_HEAP pp, ou pp est le point de programme courant pp,
   ou bien de retourner un pointeur NULL (Z_NULL);
 - mettre a jour la table heapsizes, de sorte que l'on sache que les blocs alloues a pp
   sont de taille itv', contenant les valeurs possibles de n ainsi que de toutes les
   allocations  faites au meme point de programme pp.
*)

let do_malloc vt ft info finfo sinfo notfound heapsizes cctx pp globs args restype =
	 match args with (* l'argument de malloc est cense est un entier;
			  on recupere un intervalle itv dans le cas abstrait: *)
	     [ITV_INT itv] -> let itv' = try let itv0 = Hashtbl.find heapsizes pp in
	                                         itv_union itv itv0
						 (* une chose qu'on doit faire dans malloc
						  est de mettre a jour la table heapsizes
						  des tailles possibles de tous les blocs
						  alloues au point de programme pp courant
						  (c'est specifique a malloc). *)
				         with Not_found -> itv in
			      let retval =
				  Some (ABS_ADDR (Some (ZoneTable.add Z_NULL (ITV (0.0, 0.0))
							(ZoneTable.add (Z_HEAP pp) (ITV (0.0, 0.0))
							 ZoneTable.empty)))) in
				  (* la valeur de retour de malloc est un pointeur:
				   soit le pointeur NULL, au cas ou l'allocation echoue,
				   soit le pointeur Z_HEAP pp, qui est la version abstraite
				   de tous les pointeurs possibles alloues au point de programme
				   courant pp.  Les offsets sont zero dans les deux cas
				   (en version abstraite, les intervalles [0, 0],
				   soit ITV (0.0, 0.0)). *)
				(hash_replace heapsizes pp itv';
				 (* on met a jour heapsizes, puis
				  on retourne le nouvel etat des variables globales (globs,
				  rien n'a change), la valeur de retour retval, un ensemble
				  d'adresses abstraites sur lesquels malloc a pu faire des
				  effets de bord (aucune, soit ZoneTable.empty), un
				  ensemble de signaux possiblement lances par malloc
				  (aucun, soit sigs_empty): *)
				 Some (globs, retval, Some ZoneTable.empty, sigs_empty))
	   | _ -> failwith "malloc"

let _ = Hashtbl.add external_functions "malloc" do_malloc

(*...*)
