
/*
 *	Copyright (c) 2002 by Laboratoire Spcification et Vrification (LSV),
 *	CNRS UMR 8643 & ENS Cachan.
 *	Written by Jean Goubault-Larrecq.  Not derived from licensed software.
 *
 *	Permission is granted to anyone to use this software for any
 *	purpose on any computer system, and to redistribute it freely,
 *	subject to the following restrictions:
 *
 *	1. Neither the author nor its employer is responsible for the consequences of use of
 *		this software, no matter how awful, even if they arise
 *		from defects in it.
 *
 *	2. The origin of this software must not be misrepresented, either
 *		by explicit claim or by omission.
 *
 *	3. Altered versions must be plainly marked as such, and must not
 *		be misrepresented as being the original software.
 *
 *	4. This software is restricted to non-commercial use only.  Commercial
 *		use is subject to a specific license, obtainable from LSV.
 */

#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <errno.h>

int main (int argc, char *argv[])
{
  FILE * f;

  f = fopen ("const.ml", "w");
  if (f==NULL)
    {
      fprintf (stderr, "config: could not open const.ml for writing.\n");
      exit (10);
    }
  fprintf (f, "let sig_fpe = %d\n", SIGFPE);
  fprintf (f, "let sig_segv = %d\n", SIGSEGV);
  fprintf (f, "let edom = %d\n", EDOM);
  fprintf (f, "let erange = %d\n", ERANGE);
  fprintf (f, "let nsignals = %d\n", NSIG);
  fprintf (f, "let sig_print_name s =\n    match s with\n");
  fprintf (f, "\t%d -> \"hangup\"\n", SIGHUP);
  fprintf (f, "      | %d -> \"interrupt\"\n", SIGINT);
  fprintf (f, "      | %d -> \"quit\"\n", SIGQUIT);
  fprintf (f, "      | %d -> \"illegal instruction\"\n", SIGILL);
  fprintf (f, "      | %d -> \"trap\"\n", SIGTRAP);
  fprintf (f, "      | %d -> \"abort\"\n", SIGABRT);
  fprintf (f, "      | %d -> \"bus error\"\n", SIGBUS);
  fprintf (f, "      | %d -> \"arithmetic exception\"\n", SIGFPE);
  fprintf (f, "      | %d -> \"kill\"\n", SIGKILL);
#ifdef SIGUSR1
  fprintf (f, "      | %d -> \"user1\"\n", SIGUSR1);
#endif
  fprintf (f, "      | %d -> \"segmentation violation\"\n", SIGSEGV);
#ifdef SIGUSR1
  fprintf (f, "      | %d -> \"user2\"\n", SIGUSR2);
#endif
#ifdef SIGPIPE
  fprintf (f, "      | %d -> \"pipe\"\n", SIGPIPE);
#endif
#ifdef SIGALRM
  fprintf (f, "      | %d -> \"alarm\"\n", SIGALRM);
#endif
#ifdef SIGTERM
  fprintf (f, "      | %d -> \"termination\"\n", SIGTERM);
#endif
#ifdef SIGSTKFLT
  fprintf (f, "      | %d -> \"stack flt\"\n", SIGSTKFLT);
#endif
#ifdef SIGCHLD
  fprintf (f, "      | %d -> \"child termination\"\n", SIGCHLD);
#endif
#ifdef SIGCONT
  fprintf (f, "      | %d -> \"continue\"\n", SIGCONT);
#endif
#ifdef SIGSTOP
  fprintf (f, "      | %d -> \"stop\"\n", SIGSTOP);
#endif
#ifdef SIGTSTP
  fprintf (f, "      | %d -> \"tstop\"\n", SIGTSTP);
#endif
#ifdef SIGTTIN
  fprintf (f, "      | %d -> \"ttin\"\n", SIGTTIN);
#endif
#ifdef SIGTTOU
  fprintf (f, "      | %d -> \"ttout\"\n", SIGTTOU);
#endif
#ifdef SIGURG
  fprintf (f, "      | %d -> \"urgency\"\n", SIGURG);
#endif
#ifdef SIGXCPU
  fprintf (f, "      | %d -> \"external cpu\"\n", SIGXCPU);
#endif
#ifdef SIGXFSZ
  fprintf (f, "      | %d -> \"external fsz\"\n", SIGXFSZ);
#endif
#ifdef SIGVTALRM
  fprintf (f, "      | %d -> \"virtual terminal alarm\"\n", SIGVTALRM);
#endif
#ifdef SIGPROF
  fprintf (f, "      | %d -> \"profiling\"\n", SIGPROF);
#endif
#ifdef SIGWINCH
  fprintf (f, "      | %d -> \"window close/hangup\"\n", SIGWINCH);
#endif
#ifdef SIGIO
  fprintf (f, "      | %d -> \"IO\"\n", SIGIO);
#endif
#ifdef SIGPWR
  fprintf (f, "      | %d -> \"power\"\n", SIGPWR);
#endif
  fprintf (f, "      | _ -> \"signal \" ^ (string_of_int s)\n");
  (void) fclose (f);
}
