
(*
 *	Copyright (c) 2002 by Laboratoire Spcification et Vrification (LSV),
 *	CNRS UMR 8643 & ENS Cachan.
 *	Written by Jean Goubault-Larrecq.  Not derived from licensed software.
 *
 *	Permission is granted to anyone to use this software for any
 *	purpose on any computer system, and to redistribute it freely,
 *	subject to the following restrictions:
 *
 *	1. Neither the author nor its employer is responsible for the consequences of use of
 *		this software, no matter how awful, even if they arise
 *		from defects in it.
 *
 *	2. The origin of this software must not be misrepresented, either
 *		by explicit claim or by omission.
 *
 *	3. Altered versions must be plainly marked as such, and must not
 *		be misrepresented as being the original software.
 *
 *	4. This software is restricted to non-commercial use only.  Commercial
 *		use is subject to a specific license, obtainable from LSV.
 *)

(* Projet no.5: Mettre tout ensemble: calculs sur des environnements abstraits. *)

open Const
open Arith
open Cparse
open Sigs
open Control
open Intervalle
open Itv_int
open Itv_float
open Zones

type abs_value = ITV_INT of itv (* un intervalle entier *)
    | ITV_FLOAT of (itv * bool) (* un intervalle reel, plus un booleen disant
				 si on trouve aussi des choses qui ne sont pas
				 des nombres dans les valeurs possibles (des "NaN"). *)
    | ABS_ADDR of abs_addr (* une zone d'adresses memoire abstraites. *)

module AbsEnv = Map.Make (struct
			      type t = string
			      let compare = Pervasives.compare
			  end)

type abs_env = abs_value AbsEnv.t (* : var-globale --> abs_value *)
    * abs_value AbsEnv.t (* : var-locale --> abs_value *)
    (* Note: si la variable x n'est envoyee vers rien par abs_env,
     c'est qu'elle vaut le top de son type (itv_int_top qtyp
     si qtyp est entier, itv_float_top qtyp si qtyp est flottant,
     z_top si pointeur). *)

type sig_set = int list

type abs_effect = abs_env * abs_addr * sig_set
    (* l'effet d'une instruction, c'est de retourner:
     - un nouvel environnement envoyant les variables globales/locales
     vers leurs valeurs possibles,
     - une adresse abstraite recouvrant toutes les adresses possibles qui
     ont ete ecrites par l'instruction,
     - un ensemble de signaux possibles lances par l'execution de cette instruction
     (cf. sigs.ml)
     *)

(*...*)
