
(*
 *	Copyright (c) 2002 by Laboratoire Spcification et Vrification (LSV),
 *	CNRS UMR 8643 & ENS Cachan.
 *	Written by Jean Goubault-Larrecq.  Not derived from licensed software.
 *
 *	Permission is granted to anyone to use this software for any
 *	purpose on any computer system, and to redistribute it freely,
 *	subject to the following restrictions:
 *
 *	1. Neither the author nor its employer is responsible for the consequences of use of
 *		this software, no matter how awful, even if they arise
 *		from defects in it.
 *
 *	2. The origin of this software must not be misrepresented, either
 *		by explicit claim or by omission.
 *
 *	3. Altered versions must be plainly marked as such, and must not
 *		be misrepresented as being the original software.
 *
 *	4. This software is restricted to non-commercial use only.  Commercial
 *		use is subject to a specific license, obtainable from LSV.
 *)

type itv = (* le type des intervalles, qu'ils soient entiers ou flottants *)
    ITV_EMPTY (* l'intervalle vide *)
  | ITV of float * float (* ITV (m, n), avec m<=n (intervalles reduits) *)

let itv m n = (* construit l'intervalle [m,n] *)
    if m>n
	then ITV_EMPTY
    else ITV (m, n)

let itv_union itv1 itv2 =
    match itv1, itv2 with
        ITV_EMPTY, _ -> itv2
      | _, ITV_EMPTY -> itv1
      | ITV (a1, b1), ITV (a2, b2) -> ITV (min a1 a2, max b1 b2)

let itv_inter itv1 itv2 =
    match itv1, itv2 with
        ITV_EMPTY, _ -> ITV_EMPTY
      | _, ITV_EMPTY -> ITV_EMPTY
      | ITV (a1, b1), ITV (a2, b2) -> itv (max a1 a2) (min b1 b2)

let itv_equal itv1 itv2 =
    match itv1, itv2 with
	ITV_EMPTY, ITV_EMPTY -> true
      | ITV (a1, b1), ITV (a2, b2) -> a1=a2 && b1=b2
      | _, _ -> false

let itv_in x itv =
    match itv with
        ITV_EMPTY -> false
      | ITV (m, n) -> x>=m && x<=n

let itv_subset i1 i2 =
    match i1, i2 with
	ITV_EMPTY, _ -> true
      | _, ITV_EMPTY -> false
      | ITV (a1, b1), ITV (a2, b2) -> a1>=a2 && b1<=b2

let itv_cst n = ITV (n, n)

let itv_bool possiblement_vrai possiblement_faux =
    if possiblement_vrai
	then if possiblement_faux
		 then ITV (0.0, 1.0) (* un booleen possiblement vrai et
				      possiblement faux est 0 ou 1. *)
	     else ITV (1.0, 1.0) (* un booleen possiblement vrai vaut 1. *)
    else if possiblement_faux
	then ITV (0.0, 0.0) (* un booleen possiblement faux vaut 0. *)
    else ITV_EMPTY
