
(*
 *	Copyright (c) 2002 by Laboratoire Spcification et Vrification (LSV),
 *	CNRS UMR 8643 & ENS Cachan.
 *	Written by Jean Goubault-Larrecq.  Not derived from licensed software.
 *
 *	Permission is granted to anyone to use this software for any
 *	purpose on any computer system, and to redistribute it freely,
 *	subject to the following restrictions:
 *
 *	1. Neither the author nor its employer is responsible for the consequences of use of
 *		this software, no matter how awful, even if they arise
 *		from defects in it.
 *
 *	2. The origin of this software must not be misrepresented, either
 *		by explicit claim or by omission.
 *
 *	3. Altered versions must be plainly marked as such, and must not
 *		be misrepresented as being the original software.
 *
 *	4. This software is restricted to non-commercial use only.  Commercial
 *		use is subject to a specific license, obtainable from LSV.
 *)

open Cparse
open Control
open Arg
open Analyseur
open Arith
open Intervalle
open Zones
open Glue

let input = ref stdin;;
let c_prefix = ref "a.out";;
let dumpcfg = ref false;;

(*
let rest = ref false;;
let argcr = ref 1;;
let argvr = ref [];;
*)

let basename s =
    try String.sub s 0 (String.rindex s '.')
    with Not_found -> s;;

parse [("-dumpcfg", Set dumpcfg, "dumps the control flow graph of each function in the program");
       ("-v", Unit (fun () -> verbose:=1), "report calls and returns during analysis");
       ("-v1", Unit (fun () -> verbose:=1), "report calls and returns during analysis");
       ("-v2", Unit (fun () -> verbose:=2), "report calls/returns and traces values during analysis")
	   (*
       ("--", Rest (fun s -> (rest := true;
			      incr argc;
			      argv := s:: !argv)),
	"pass remaining arguments as arguments to main()")
	    *)
	   ]
      (fun s -> (c_prefix := basename s;
		 Ctype.context := [!c_prefix];
		 input := (Unix.open_process_in ("cpp \"" ^ (String.escaped s) ^ "\"") : in_channel)))
	   "analyzes a C program";;

  let lexbuf = Lexing.from_channel (!input) in
  let c = Ctab.translation_unit Clex.ctoken lexbuf in
      (Error.flush_error ();
       let vt, ft = cgs_of_program c in
	   (Error.flush_error ();
	    if !dumpcfg
		then let out = open_out (!c_prefix ^ ".cfg") in
		    (pr_cgs out (vt, ft); close_out out)
	    else ();
	    let info = (Hashtbl.create 128 : abs_info) in
	    let finfo = (Hashtbl.create 128 : function_info) in
	    let sigs = (Hashtbl.create 128 : sig_info) in
	    let not_found = (Hashtbl.create 128 : functions_not_found) in
	    let heapsizes = (Hashtbl.create 128 : heap_size_info) in
		(*
	    let argc = !argcr in
	    let argv = !cprefix :: List.rev (!argvr) in
		 *)
	    let args = [ ITV_INT (itv 1.0 (type_int.it_max));
			 ABS_ADDR (z_make (Z_GLOBAL ("__argv",
						     itv 0.0 (type_int.it_max))));
			 ABS_ADDR (z_make (Z_GLOBAL ("environs",
						     itv 0.0 (type_int.it_max))))
			 ] in
(*!!!should check that main() has type
 int main (int argc, char **argv, char **environs) [not sure about environs],
 and that environs has the right type. *)
	    let _ =
		analyseur vt ft info finfo sigs not_found heapsizes CallContext.empty None
		          AbsEnv.empty "main" args ((false, false), TINT) in
		(Report.code_mort ft info;
		 Report.sigs ft sigs;
		 flush stdout)
		)
	   );;

