
(*
 *	Copyright (c) 2002 by Laboratoire Spcification et Vrification (LSV),
 *	CNRS UMR 8643 & ENS Cachan.
 *	Written by Jean Goubault-Larrecq.  Not derived from licensed software.
 *
 *	Permission is granted to anyone to use this software for any
 *	purpose on any computer system, and to redistribute it freely,
 *	subject to the following restrictions:
 *
 *	1. Neither the author nor its employer is responsible for the consequences of use of
 *		this software, no matter how awful, even if they arise
 *		from defects in it.
 *
 *	2. The origin of this software must not be misrepresented, either
 *		by explicit claim or by omission.
 *
 *	3. Altered versions must be plainly marked as such, and must not
 *		be misrepresented as being the original software.
 *
 *	4. This software is restricted to non-commercial use only.  Commercial
 *		use is subject to a specific license, obtainable from LSV.
 *)

(* Projet 8: afficher des alertes correspondant aux bugs trouves
 (dereferencement de pointeur hors de sa zone, code mort
  notamment) . *)

open Error
open Const
open Sigs
open Control
open Prglue
open Analyseur

(*
let merge_sort cmp =
    let rec merge l1 l2 =
	match l1, l2 with
	    [], _ -> l2
	  | _, [] -> l1
	  | a1::r1, a2::r2 ->
	    let n = cmp a1 a2 in
		if n>0
		    then a2 :: merge l1 r2
		else if n<0
		    then a1 :: merge r1 l2
		else a1 :: merge r1 r2 in
    let rec split l =
	match l with
	    [] -> [], []
	  | [a] -> l, []
	  | a::b::r -> let l1, l2 = split r in
	                   a::l1, b::l2 in
    let rec sort l =
	let l1, l2 = split l in
	    merge (sort l1) (sort l2) in
	sort
*)

type 'a heap = H0 | H2 of 'a * 'a heap * 'a heap
exception EmptyHeap

let heap_sort cmp =
    let rec up w h =
	match h with
	    H0 -> H2 (w, H0, H0)
	  | H2 (v, p, q) ->
	    let n = cmp w v in
		if n>0
		    then H2 (w, up v q, p)
		else if n<0
		    then H2 (v, up w q, p)
		else h in
    let rec removeleft h =
	match h with
	    H0 -> raise EmptyHeap
	  | H2 (w, H0, _) -> w, H0
	  | H2 (w, p, q) ->
	    let x, p' = removeleft p in
		x, H2 (w, q, p') in
    let rec siftdown x h1 h2 =
	match h1, h2 with
	    H0, _ -> H2 (x, H0, H0)
	  | (H2 (y, _, _) as p), H0 ->
	    let n = cmp x y in
		if n>=0
		    then H2 (x, p, H0)
		else H2 (y, H2 (x, H0, H0), H0)
	  | (H2 (y, p1, q1) as p), (H2 (z, p2, q2) as q) ->
	    if cmp x y >=0 && cmp x z >=0
		then H2 (x, p, q)
	    else if cmp y z >=0
		then H2 (y, siftdown x p1 q1, q)
	    else H2 (z, p, siftdown x p2 q2) in
    let rec deletemin h =
	match h with
	    H0 -> raise EmptyHeap
	  | H2 (w, H0, q) -> q
	  | H2 (w, p, q) ->
	    let x, p' = removeleft p in
		siftdown x q p' in
    let rec mklist h acc =
	match h with
	    H0 -> acc
	  | (H2 (w, _, _) as p) -> mklist (deletemin p) (w::acc) in
    let rec putaway l h =
	match l with
	    [] -> h
	  | a::r -> putaway r (up a h) in
    let sort l = mklist (putaway l H0) [] in
	sort

let invert_hash info =
    let rinfo = Hashtbl.create 128 in
	(Hashtbl.iter (fun ctx loc_info ->
			   Hashtbl.iter (fun pp env ->
					     let loc_rinfo =
						 (try Hashtbl.find rinfo pp
						     with Not_found ->
							  let lr = Hashtbl.create 1 in
							      (Hashtbl.add rinfo pp lr; lr)) in
						 Hashtbl.add loc_rinfo ctx env
						 ) loc_info
			   ) info;
	 rinfo)

let print_lines l =
    let dump_range file m n =
	(print_string "  file \"";
	 print_string (String.escaped file);
	 print_string "\", line";
	 (if m=n
	      then (print_string " ";
		    print_int m)
	  else (print_string "s ";
		print_int m;
		print_string "-";
		print_int n));
	      print_string ";\n") in
    let rec find_range file m n l =
	match l with
	    [] -> dump_range file m n
	  | (file', line') :: r ->
	    if file=file' && line'=n+1
		then find_range file m line' r
	    else (dump_range file m n;
		  find_range file' line' line' r) in
    match l with
	[] -> ()
      | (file,line)::r -> find_range file line line r

let code_mort (ft : fun_table) (info : abs_info) =
    (let rinfo = invert_hash info in
    let deadfuns = ref [] in
    Hashtbl.iter (fun f (st, loc, rest, entryp, exitp, cg, n, typenv, abbrvs) ->
		      try (ignore (Hashtbl.find rinfo (f, entryp));
			   let deads = Hashtbl.create 128 in
			   let deadl = ref [] in
			   let somedead = ref false in
			       (for i=1 to n do
				try ignore (Hashtbl.find rinfo (f, i))
				   with Not_found ->
					(
prerr_string "dead: ";
prerr_int i;
prerr_string "; ";
try let succs = Hashtbl.find cg i in
					 Hashtbl.iter (fun tgt (loc, _) ->
							   let (file, line, _) = loc_start loc in
							       (try ignore (Hashtbl.find deads (file, line))
								   with Not_found ->
									(Hashtbl.add deads (file, line) ();
									 deadl := (file,line) :: !deadl);
									somedead := true)
							       ) succs
			                 with Not_found -> ())
					done;
					if !somedead then
					    (print_string "Dead code in '";
					     print_string f;
					     print_string "':\n";
					     print_lines
					     (heap_sort (fun (file1,line1) (file2,line2) ->
							     let n = compare file1 file2 in
								 if n=0
								     then compare line1 line2
								 else n
								     ) (!deadl))
					     )
					    ))
			       with Not_found -> deadfuns := f :: !deadfuns
		      ) ft;
    match !deadfuns with
	[] -> ()
      | df -> (print_string "Unreachable functions:\n ";
	       List.iter (fun f -> (print_string " "; print_string f)) df;
	       print_string "\n")
	 )

let print_sigs (sigs : sig_set) =
    if sigs=all_sigs
	then print_string " all signals"
    else let sep = ref " " in
	List.iter (fun s ->
		       (print_string (!sep);
			sep := ", ";
			print_string (sig_print_name s))
		       ) sigs

let sigs (ft : fun_table) (sigs : sig_info) =
    let rsigs = invert_hash sigs in
	Hashtbl.iter (fun ((file, line, _, _, _), _) ctxts ->
			  print_string "File \"";
			  print_string file;
			  print_string "\", line ";
			  print_int line;
			  print_string ":\n";
			  Hashtbl.iter (fun ctx s ->
					    print_string "  may raise";
					    print_sigs s;
					    (match ctx with
						 None -> ()
					       | Some pp ->
						 print_string " when called from ";
						 output_loc_pp stdout ft pp);
					    print_string "\n"
					    ) ctxts
			  ) rsigs
