
(*
 *	Copyright (c) 2002 by Laboratoire Spcification et Vrification (LSV),
 *	CNRS UMR 8643 & ENS Cachan.
 *	Written by Jean Goubault-Larrecq.  Not derived from licensed software.
 *
 *	Permission is granted to anyone to use this software for any
 *	purpose on any computer system, and to redistribute it freely,
 *	subject to the following restrictions:
 *
 *	1. Neither the author nor its employer is responsible for the consequences of use of
 *		this software, no matter how awful, even if they arise
 *		from defects in it.
 *
 *	2. The origin of this software must not be misrepresented, either
 *		by explicit claim or by omission.
 *
 *	3. Altered versions must be plainly marked as such, and must not
 *		be misrepresented as being the original software.
 *
 *	4. This software is restricted to non-commercial use only.  Commercial
 *		use is subject to a specific license, obtainable from LSV.
 *)

(* Projet no.4: Calculs de zones memoires pointees par des pointeurs. *)

open Arith
open Sizeof
open Intervalle
open Itv_int

exception Zone (* lancee pour retourner top. *)

type zone = Z_NULL (* le pointeur NULL *)
   | Z_GLOBAL of string * itv (* Z_GLOBAL (x, [m, n]): une zone reservee
			       pour la variable globale de nom x,
			       qui s'etend sur un nombre d'octets
			       compris entre m et n. *)
   | Z_HEAP of Control.ppoint (* Z_HEAP (pp): une zone de
				memoire allouee au point de programme pp.
				On ne garde pas sa taille ici. *)
   | Z_STACK of string * itv * string * Control.loc_ppoint option
     (* Z_STACK (x, [m, n], f, pp): une zone de memoire allouee pour
      la variable locale x, s'etendant sur de m a n octets.
      La variable locale x est definie dans la fonction f, qui est
      appelee depuis le point de programme local pp
      (ou None si f est le point d'entree principal main du programme). *)
   | Z_STRING of string (* une zone reservee pour la constante chaine
			 fournie en argument; si n est la longueur de
			 la chaine, le type de cette constante est
			 const unsigned char [n+1]; les ecritures dans
			 cette zone echouent. *)
   | Z_FUN of string (* Z_FUN f: une zone reservee pour le code de la
		      fonction f; on ne sait pas sur combien d'octets
		      ca s'etend; les ecritures dans cette zone echouent. *)

module ZoneTable = Map.Make (struct
				  type t = zone
				  let compare = Pervasives.compare
			      end)
			      
type abs_addr = (* une adresse abstraite est en gros une liste de zones
		 possibles.  Comme on peut ajouter n'importe quel entier
		 a un pointeur C, il faudra aussi dire pour chaque zone
		 quels sont les decalages [offsets] possibles de l'adresse
		 par rapport au debut de la zone.
		   En consequence, une adresse abstraite est une table qui
		 a des zones associe des intervalles non vides d'offsets...
		   ou bien None, signifiant un pointeur qui peut aller
		 n'importe ou (top). *)
    itv ZoneTable.t option (* la table zone -> intervalle non vide, ou None. *)

type heap_size_info = (Control.ppoint, itv) Hashtbl.t

(*...*)
